// Main program written in C++ that calls a Fortran DLL routine.
/* Note the following items:
   - Fortran character arguments have a hidden length
     argument following the address
   - Routine name must be in uppercase to match Fortran.
*/


#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "stdafx.h"


typedef void (CALLBACK* DLL_ROUT)(int *nv,
    double x[101],
        double sg[101],
        double xxs[101],
        double rho[101][101],
        double *t,
        int *nt,
        double *value);
                        
int main (int argc, char *argv[])
{
     HINSTANCE hdll;	
	int i, j, nv;
    double x[101];
    double sg[101];
    double xxs[101];
        double rho[101][101];
  DLL_ROUT pointf;
//
//   Assign nv: number of spatial variables. The intger nv is the number of underlyings 
//   considered (Bound: 1<nv<102). 
  nv=2;

// Assign t: time to maturity (Bound: 0<t<1) where the probability 
//density must  be evaluated.

        double t=0.5;
      

/* Inizialize value */
        double value=0;
       
		hdll=LoadLibrary("UnivPM_MathFinSolver1_1.dll");
	if(!hdll)
	{ printf("Read Error");
	return(0);
	}
	if(!(pointf = (DLL_ROUT)GetProcAddress(hdll,"DLL_ROUT")))
	{ printf("error");
	return(0);
	}

//  Assign the volatilities sg[0],sg[1],...,sg[nv-1]. Bound: 0.005<sg[i]<0.6, i=0,1,...,nv-1. 
// For i=0,1,...,nv-1 sg[i] is the volatility associated to the (i+1)-th underlying.
     	sg[0]=0.5;
	sg[1]=0.25;
	for (i=2; i<nv; i++)
	 {
     	 sg[i]=0.15;
	 }

//  Assign the correlation matrix rho[i],[j], i=0,1,...,nv-2,
// j=i+1,i+2,...,nv-1 (Bound: -1<rho[i][j]<1, i=0,1,...,nv-2, j=i+1,i+2,...,nv-1).
// For  i=0,1,...,nv-2, j=i+1,i+2,...,nv-1 rho[i][j] is the correlation coefficient between the 
// square roots of the (i+1)-th underlying and of the (j+1)-th underlying.
   for( i=0; i<nv-1; i++){
                for (j=i+1; j<nv; j++)
                        rho[i][j]=0.1;
        }
// The remaining correlation coefficients are fixed as follows:
		 rho[0][0]=1.0;
        for( i=1; i<nv; i++){
			  rho[i][i]=1.0;
                for (j=0; j<i; j++)
                        rho[i][j]=rho[j][i];
        }
     
// Assign x[], the vector containing the point whose components are 
// the prices at maturity time where the transition probability density  
// must be evaluated.
// For i=0,1,..,nv-1 x[i] is the price at maturity time of the (i+1)-th underlying.
// Example:

		    for( i=0; i<nv; i++){
                x[i]=1.1+0.001+(i)*0.05/(float)nv;
/* Assign the initial point xxs[]  */
                xxs[i]=0.95;
}

// Assign xxs[], the vector containing the point whose components are 
// the prices at time to maturity t>0 where the transition probability 
// density must be evaluated.
// For i=0,1,..,nv-1 xxs[i] is the price at time to maturity t of the (i+1)-th underlying.

			for( i=0; i<nv; i++){
                              xxs[i]=0.95;
        }


     
/*
!! dll parameters:
!!   nv: number of spatial variables. The integer nv is the number of underlyings considered.
!!  x[]: the vector containing the point whose components are the prices at maturity time where the 
!!       transition probability density  must be evaluated.
!!xxs[]: the vector containing the point whose components are the prices at time to maturity t>0 where 
!!       the transition probability density must be evaluated.
!!   sg: array of dimension nv containing the volatilities (assigned by the user in the main).
!!  rho: correlation matrix of dimension nv x nv (assigned by the user in the main)
!!    t: time to maturity where the transition probability density must be evaluated.
!!       (assigned by the user in the main).
!!   nt: number of terms in the time series expansion in time of the first order correction
!!       term (see formula (7.1) Section 7). The largest value of nt feasible is 9. The strongly
!!       recommended value is nt=2.
*/
			  int nt=2;
    // dll call: 

   pointf(&nv,x,sg,xxs,rho,&t,&nt,&value);
        printf("value:%le\n",value);

   FreeLibrary(hdll);
}
