! Main program written in Fortran that calls a Fortran DLL routine.
! Note the following items:
! - the dll import specification is required
! - routine name must be in uppercase to match Fortran.
!
! Declare an interface block to the routine to be called.
	interface
	  subroutine rtn_DLL_ROUT(nv,x,sg,xxs,rho,t,nt,value)	   	 
		real*8 xxs(101), x(101), sg(101), rho(101,101), t, value
		integer nv, nt
	  end subroutine
	end interface

	call beginning()
end


subroutine beginning()
  integer mxdim, ndim, nt, nv
  parameter(mxdim=101)
  real*8  resf,  value
		
  integer i, j
  real*8 x(101),rho(101,101), sg(101), xxs(101), t
  real*8 z(101), min
  common/areadati/rho,sg,x,t

! Assign ndim: number of spatial variables (Bound: 1<ndim<102). 
! The integer ndim is the number of underlyings considered.
	    ndim=2
		if (ndim.ge.101) then 
			write(*,*) 'Error: insert the variables number must be less than 102'
			stop
		endif

! Assign t: time to maturity (Bound: 0<t<1) where the probability 
! density must  be evaluated.

		t=0.5

                if(t.GE.1.0d0.OR.t.LE.0.0) then
			write(*,*) 'Error in time to maturity t: 0<t<1'
			stop
		endif

!  Assign the volatilities sg(1),sg(2),...,sg(ndim). 0.005<sg(i)<0.6, i=1,2,...,ndim.
                 sg(1)=0.5
        	 sg(2)=0.25
                 do i=3, ndim
                   sg(i)=0.15
                 enddo
				 

!  Assign the correlation matrix rho(i,j), i=1,2,...,ndim-1,
!  j=i+1,i+2,...,ndim (Bound: -1<rho(i,j)<1, i=1,2,...,ndim-1, j=i+1,i+2,...,ndim).
                 do i=1, ndim-1
                   do j=i+1, ndim
                     rho(i,j)=0.1
                   enddo
                 enddo
! The remaining correlation coefficients are fixed as follows:
                 rho(1,1)=1.d0
                 do i=2, ndim
                   rho(i,i)=1.d0
                   do j=1, i-1
                     rho(i,j)=rho(j,i)
                   enddo
                 enddo

! Assign x(), the vector containing the point whose components are 
! the prices at maturity time where the transition probability density  
! must be evaluated.
! Example:
                 do i=1, ndim
                   x(i)=1.1+0.001+(i-1)*0.05/dfloat(ndim)
                 enddo

! Assign xxs(), the vector containing the point whose components are 
! the prices at time to maturity t>0 where the transition probability 
! density must be evaluated.


 
                 do i=1, ndim
                   xxs(i)=0.95
                 enddo

!     
!
!! dll parameters:
!!   nv: number of spatial variables  (nv=ndim).
!!  x(): the vector containing the point whose components are the prices at maturity time where the 
!!       transition probability density  must be evaluated.
!
!!xxs(): the vector containing the point whose components are the prices at time to maturity t>0 where 
!!       the transition probability density must be evaluated.
!!   sg: array of dimension nv containing the volatilities (assigned by the user in the main).
!!  rho: correlation matrix of dimension nv x nv (assigned by the user in the main)
!!    t: time to maturity where the transition probability density must be evaluated.
!!       (assigned by the user in the main).
!!   nt: number of terms in the time series expansion in time of the first order correction
!!       term (see formula (7.1) Section 7). The largest value of nt feasible is 9. The strongly
!!       recommended value is nt=2.
      nt=2
      nv=ndim 			 
				                
! dll call:
          call calcp(nv,x,sg,xxs,rho,t, nt, resf)
          write(*,*) 'value:', resf
! resf: returned by the dll contains the required value of the transition probability density.

end






subroutine calcp(nv,x,sg,xxs,rho,t,nt,value)
!use dfwin
	use kernel32
!dll variables:
	real*8 xxs(101), x(101), sg(101), rho(101,101), t, value
    integer nv, nt, jjj, iii

	 
!to call dll:
	pointer (p,i)
	pointer (q,rtn_DLL_ROUT)
	logical status

! First, locate the dll and load it into memory.
    p = loadlibrary("UnivPM_MathFinSolver1_1.dll"C)
	if (p == 0) then
		type *, "Error occurred opening testsub.dll"
		type *, "Program aborting"
		goto 1000
	endif

! Set up a pointer to the routine of interest.
    q = getprocaddress(p, "DLL_ROUT"C)
	if (q == 0) then
		type *, "Error occurred in finding DLL_ROUT in testsub.dll"
		type *, "Program aborting"
		goto 1000
	endif

!Initialize the parameters of DLL_ROUT.
	value=0
	
    

! Now, the routine can simply be called as follows:


    call rtn_DLL_ROUT(nv,x,sg,xxs,rho,t,nt,value)
	status = freelibrary(p)

1000 continue

end 

	 
